/**
*** \file calcephem_test.cpp
*** \brief Module 'calcephem' unit test file.
*** \details Unit test file for the 'calcephem' module.
**/

/*
** This file is part of moonphase.
** Copyright (C) 2014-2023 Alan Wise <awisesoftware@gmail.com>.
**
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


/**
*** \internal
*** \brief calcephem_test.cpp identifier.
*** \details Identifier for calcephem_test.cpp.
*** \endinternal
**/
#define   CALCEPHEM_TEST_CPP

/* GCOVR_ EXCL_START LCOV_ EXCL_START */


/****
*****
***** INCLUDES
*****
****/

/**
*** \brief Master test suite name.
*** \details The name of the master test suite.
**/
#define   BOOST_TEST_MODULE   "CalcEphem"
#include  <boost/test/included/unit_test.hpp>

#include  "calcephem.c"


/****
*****
***** DEFINES
*****
****/

#define   DOWSTRING_SIZE  (1024)   // Should be PLENTY.


/****
*****
***** DATA TYPES
*****
****/

typedef struct
{
  int Year;
  int Month;
  int Day;
  int DOWIndex;
  char DOWString[DOWSTRING_SIZE];
} DOWTEST_T;

typedef struct
{
  int Year;
  int Month;
  int Day;
  int DOY;
} DOYTEST_T;

typedef struct
{
  int Year;
  int Month;
  int Day;
  double UniversalTime;
  double JulianDate;
} JDTEST_T;

typedef struct
{
  double T;
  double RA;
  double Dec;
} MINIMOONTEST_T;

typedef struct
{
  int Year;
  int Month;
  int Day;
  double Hour;
  double UTRise;
  double UTSet;
} MOONRISETEST_T;

typedef struct
{
  double Input;
  double Output;
} TEST_T;


/****
*****
***** CLASSES
*****
****/


/****
*****
***** PROTOTYPES
*****
****/


/****
*****
***** DATA
*****
****/

static TEST_T const f_pAngle2PITestCases[]=
{
  { -6.283185, 0        },
  { -5.497787, 0.785398 },
  { -4.712389, 1.570796 },
  { -3.926990, 2.356194 },
  { -3.141593, 3.141593 },
  { -2.356194, 3.926990 },
  { -1.570796, 4.712389 },
  { -0.785398, 5.497787 },
  {  0       , 0        },
  {  0.785398, 0.785398 },
  {  1.570796, 1.570796 },
  {  2.356194, 2.356194 },
  {  3.141593, 3.141593 },
  {  3.926990, 3.926990 },
  {  4.712389, 4.712389 },
  {  5.497787, 5.497787 },
  {  6.283185, 6.283185 },  // Should be 0?
  {  7.068583, 0.785398 },
  {  7.853982, 1.570796 },
  {  8.639380, 2.356194 },
  {  9.424778, 3.141593 },
  { 10.210176, 3.926990 },
  { 10.995574, 4.712389 },
  { 11.780972, 5.497787 },
  { 12.566371, 0        }
};
static int const f_Angle2PITestCaseCount=
  sizeof(f_pAngle2PITestCases)/sizeof(*f_pAngle2PITestCases);

static TEST_T const f_pAngle360TestCases[]=
{
  { -360, 360 },            // Should be 0?
  { -315,  45 },
  { -270,  90 },
  { -225, 135 },
  { -180, 180 },
  { -135, 225 },
  {  -90, 270 },
  {  -45, 315 },
  {    0,   0 },
  {   45,  45 },
  {   90,  90 },
  {  135, 135 },
  {  180, 180 },
  {  225, 225 },
  {  270, 270 },
  {  315, 315 },
  {  360, 360 },            // Should be 0?
  {  405,  45 },
  {  450,  90 },
  {  495, 135 },
  {  540, 180 },
  {  585, 225 },
  {  630, 270 },
  {  675, 315 },
  {  720,   0 }
};
static int const f_Angle360TestCaseCount=
  sizeof(f_pAngle360TestCases)/sizeof(*f_pAngle360TestCases);

static DOWTEST_T const f_pDayofWeekTestCases[]=
{
  { 2000, 1, 1, 6, "Saturday"  },
  { 2000, 1, 2, 0, "Sunday"    },
  { 2000, 1, 3, 1, "Monday"    },
  { 2000, 1, 4, 2, "Tuesday"   },
  { 2000, 1, 5, 3, "Wednesday" },
  { 2000, 1, 6, 4, "Thursday"  },
  { 2000, 1, 7, 5, "Friday"    }
};
static int const f_DayofWeekTestCaseCount=
  sizeof(f_pDayofWeekTestCases)/sizeof(*f_pDayofWeekTestCases);

static DOYTEST_T const f_pDayofYearTestCases[]=
{
  { 2000,  1, 1,   1 },
  { 2000,  2, 1,  32 },
  { 2000,  3, 1,  61 },
  { 2000,  4, 1,  92 },
  { 2000,  5, 1, 122 },
  { 2000,  6, 1, 153 },
  { 2000,  7, 1, 183 },
  { 2000,  8, 1, 214 },
  { 2000,  9, 1, 245 },
  { 2000, 10, 1, 275 },
  { 2000, 11, 1, 306 },
  { 2000, 12, 1, 336 },
  { 2001,  3, 1,  60 }    // Non leap year
};
static int const f_DayofYearTestCaseCount=
  sizeof(f_pDayofYearTestCases)/sizeof(*f_pDayofYearTestCases);

static TEST_T const f_pFracTestCases[]=
{
  { -1.0, 0.0 },
  { -0.9, 0.1 },
  { -0.8, 0.2 },
  { -0.7, 0.3 },
  { -0.6, 0.4 },
  { -0.5, 0.5 },
  { -0.4, 0.6 },
  { -0.3, 0.7 },
  { -0.2, 0.8 },
  { -0.1, 0.9 },
  {  0.0, 0.0 },
  {  0.1, 0.1 },
  {  0.2, 0.2 },
  {  0.3, 0.3 },
  {  0.4, 0.4 },
  {  0.5, 0.5 },
  {  0.6, 0.6 },
  {  0.7, 0.7 },
  {  0.8, 0.8 },
  {  0.9, 0.9 },
  {  1.0, 0.0 }
};
static int const f_FracTestCaseCount=
  sizeof(f_pFracTestCases)/sizeof(*f_pFracTestCases);

static TEST_T const f_pHour24TestCases[]=
{
  { -24, 24 },              // Should be 0?
  { -20,  4 },
  { -16,  8 },
  { -12, 12 },
  {  -8, 16 },
  {  -4, 20 },
  {   0,  0 },
  {   4,  4 },
  {   8,  8 },
  {  12, 12 },
  {  16, 16 },
  {  20, 20 },
  {  24, 24 },              // Should be 0?
  {  28,  4 },
  {  32,  8 },
  {  36, 12 },
  {  40, 16 },
  {  44, 20 },
  {  48,  0 }
};
static int const f_Hour24TestCaseCount=
  sizeof(f_pHour24TestCases)/sizeof(*f_pHour24TestCases);

static JDTEST_T const f_pjdTestCases[]=
{
  {   -1, 12, 31,  0,        1721422.5 },
  {    1,  1,  1,  0,        1721423.5 },
  {    1,  2,  1,  0,        1721454.5 },
  { 1582, 10,  4,  0,        2299159.5 },
  { 1582, 10, 15,  0,        2299160.5 },
  { 1999, 12, 31, 23.999722, 2451544.5 }
};
static int const f_jdTestCaseCount=
  sizeof(f_pjdTestCases)/sizeof(*f_pjdTestCases);

static MINIMOONTEST_T const f_pMiniMoonTestCases[]=
{
  // First value calculated using: (jd(year,month,day,UT)-2451545.0)/36525.0
  { 0.199986,  23.227, -10.01 },  // 2020/01/01
  { 0.210007,   8.392,  22.97 },  // 2021/01/01
  { 0.220000,  16.944, -24.04 },  // 2022/01/01
  { 0.229993,   2.119,  12.03 }   // 2023/01/01
};
static int const f_MiniMoonTestCaseCount=
  sizeof(f_pMiniMoonTestCases)/sizeof(*f_pMiniMoonTestCases);

static MOONRISETEST_T const f_pMoonRiseTestCases[]=
{
  { 2000,1,1,0,2.466667,13.75 },
};
static int const f_MoonRiseTestCaseCount=
  sizeof(f_pMoonRiseTestCases)/sizeof(*f_pMoonRiseTestCases);

static TEST_T const f_pSineTestCases[]=
{
  { -1.0,  0.000000 },
  { -0.9,  0.587785 },
  { -0.8,  0.951057 },
  { -0.7,  0.951057 },
  { -0.6,  0.587785 },
  { -0.5,  0.000000 },
  { -0.4, -0.587785 },
  { -0.3, -0.951057 },
  { -0.2, -0.951057 },
  { -0.1, -0.587785 },
  {  0.0,  0.000000 },
  {  0.1,  0.587785 },
  {  0.2,  0.951057 },
  {  0.3,  0.951057 },
  {  0.4,  0.587785 },
  {  0.5,  0.000000 },
  {  0.6, -0.587785 },
  {  0.7, -0.951057 },
  {  0.8, -0.951057 },
  {  0.9, -0.587785 },
  {  1.0,  0.000000 }
};
static int const f_SineTestCaseCount=
  sizeof(f_pSineTestCases)/sizeof(*f_pSineTestCases);


/****
*****
***** VARIABLES
*****
****/


/****
*****
***** FUNCTIONS
*****
****/

double HMS2Deg(double H,double M,double S)
{
  bool Positive;
  double Value;


  Positive=(H>=0);
  Value=fabs(H)+M/60.0+S/3600.0;
  Value*=15;
  if (!Positive)
    Value=-Value;

  return(Value);
}

double DMS2Deg(double D,double M,double S)
{
  bool Positive;
  double Value;


  Positive=(D>=0);
  Value=fabs(D)+M/60.0+S/3600.0;
  if (!Positive)
    Value=-Value;

  return(Value);
}


/****
*****
***** TESTS
*****
****/

BOOST_AUTO_TEST_SUITE(Module__calcephem)
  BOOST_AUTO_TEST_SUITE(Function__angle2pi)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_Angle2PITestCaseCount;Index++)
      {
        BOOST_TEST(angle2pi(f_pAngle2PITestCases[Index].Input)==
            f_pAngle2PITestCases[Index].Output,
            boost::test_tools::tolerance(0.00001));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__angle360)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_Angle360TestCaseCount;Index++)
      {
        BOOST_TEST(angle360(f_pAngle360TestCases[Index].Input)==
            f_pAngle360TestCases[Index].Output,
            boost::test_tools::tolerance(0.00001));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__DayofWeek)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      char pDOWString[DOWSTRING_SIZE];


      for(int Index=0;Index<f_DayofWeekTestCaseCount;Index++)
      {
        BOOST_TEST(
            DayofWeek(f_pDayofWeekTestCases[Index].Year,
            f_pDayofWeekTestCases[Index].Month,
            f_pDayofWeekTestCases[Index].Day,pDOWString)==
            f_pDayofWeekTestCases[Index].DOWIndex);
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__DayofYear)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_DayofYearTestCaseCount;Index++)
      {
        BOOST_TEST(
            DayofYear(f_pDayofYearTestCases[Index].Year,
            f_pDayofYearTestCases[Index].Month,
            f_pDayofYearTestCases[Index].Day)==f_pDayofYearTestCases[Index].DOY);
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__frac)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_FracTestCaseCount;Index++)
      {
        BOOST_TEST(
            frac(f_pFracTestCases[Index].Input)==f_pFracTestCases[Index].Output,
            boost::test_tools::tolerance(0.00001));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__hour24)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_Hour24TestCaseCount;Index++)
      {
        BOOST_TEST(hour24(f_pHour24TestCases[Index].Input)==
            f_pHour24TestCases[Index].Output,
            boost::test_tools::tolerance(0.00001));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__jd)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_jdTestCaseCount;Index++)
      {
        BOOST_TEST(jd(f_pjdTestCases[Index].Year,f_pjdTestCases[Index].Month,
            f_pjdTestCases[Index].Day,f_pjdTestCases[Index].UniversalTime)==
            f_pjdTestCases[Index].JulianDate,boost::test_tools::tolerance(0.01));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__MiniMoon)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_MiniMoonTestCaseCount;Index++)
      {
        double RA;
        double Dec;


        MiniMoon(f_pMiniMoonTestCases[Index].T,&RA,&Dec);
        BOOST_TEST(f_pMiniMoonTestCases[Index].RA==RA,
            boost::test_tools::tolerance(0.05));
        BOOST_TEST(f_pMiniMoonTestCases[Index].Dec==Dec,
            boost::test_tools::tolerance(0.05));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__MoonRise)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_MoonRiseTestCaseCount;Index++)
      {
        double Rise;
        double Set;


        MoonRise(f_pMoonRiseTestCases[Index].Year,f_pMoonRiseTestCases[Index].Month,f_pMoonRiseTestCases[Index].Day,f_pMoonRiseTestCases[Index].Hour,&Rise,&Set);
        BOOST_TEST(f_pMoonRiseTestCases[Index].UTRise==Rise,boost::test_tools::tolerance(0.05));
        BOOST_TEST(f_pMoonRiseTestCases[Index].UTSet==Set,boost::test_tools::tolerance(0.05));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
  BOOST_AUTO_TEST_SUITE(Function__sine)
    BOOST_AUTO_TEST_CASE(Test_00)
    {
      for(int Index=0;Index<f_SineTestCaseCount;Index++)
      {
        BOOST_TEST(
            sine(f_pSineTestCases[Index].Input)==f_pSineTestCases[Index].Output,
            boost::test_tools::tolerance(0.00001));
      }
    }
  BOOST_AUTO_TEST_SUITE_END()
BOOST_AUTO_TEST_SUITE_END()

/* LCOV_ EXCL_STOP GCOVR_ EXCL_STOP */


#undef    CALCEPHEM_TEST_CPP


/**
*** calcephem_test.cpp
**/
