/**
*** \file moonanimation.cpp
*** \brief Module 'moonanimation' implementation file.
*** \details Implementation file for the 'moonanimation' module.
*** \todo Read other formats (i.e. formats with animations, etc.)
**/

/*
** This file is part of moonphase.
** Copyright (C) 2014-2023 Alan Wise <awisesoftware@gmail.com>.
**
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation, either version 3 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


/**
*** \internal
*** \brief moonanimation.cpp identifier.
*** \details Identifier for moonanimation.cpp.
*** \endinternal
**/
#define   MOONANIMATION_CPP


/****
*****
***** INCLUDES
*****
****/

#include  "moonanimation.h"
#if       defined(DEBUG_MOONANIMATION_CPP)
#if       !defined(USE_DEBUGLOG)
#define   USE_DEBUGLOG
#endif    /* !defined(USE_DEBUGLOG) */
#endif    /* defined(DEBUG_MOONANIMATION_CPP) */
#include  "debuglog.h"
#include  "messagelog.h"


/****
*****
***** DEFINES
*****
****/


/****
*****
***** DATA TYPES
*****
****/


/****
*****
***** CLASSES
*****
****/


/****
*****
***** PROTOTYPES
*****
****/


/****
*****
***** DATA
*****
****/


/****
*****
***** VARIABLES
*****
****/


/****
*****
***** FUNCTIONS
*****
****/

ERRORCODE_T MoonAnimation_GetFrame(
    MOONANIMATION_T const *pMoonAnimation,unsigned Index,QPixmap const **ppFrame)
{
  ERRORCODE_T ErrorCode;


  DEBUGLOG_Printf3(
      "MoonAnimation_GetFrame(%p,%u,%p)",pMoonAnimation,Index,ppFrame);
  DEBUGLOG_LogIn();

  /* Parameter checking. */
  if ( (pMoonAnimation==NULL) || (ppFrame==NULL) )
  {
    ErrorCode=ERRORCODE_NULLPARAMETER;
    MESSAGELOG_LogError(ErrorCode);
  }
  else if (Index>static_cast<unsigned>(pMoonAnimation->Images.count()))
  {
    ErrorCode=ERRORCODE_INVALIDPARAMETER;
    MESSAGELOG_LogError(ErrorCode);
  }
  else
  {
    *ppFrame=&((pMoonAnimation->Images)[static_cast<int>(Index)]);
    ErrorCode=ERRORCODE_SUCCESS;
  }

  DEBUGLOG_LogOut();
  return(ErrorCode);
}

ERRORCODE_T MoonAnimation_GetFrameCount(MOONANIMATION_T const *pMoonAnimation)
{
  ERRORCODE_T ErrorCode;


  DEBUGLOG_Printf1("MoonAnimation_GetFrameCount(%p)",pMoonAnimation);
  DEBUGLOG_LogIn();

  /* Parameter checking. */
  if (pMoonAnimation==NULL)
  {
    ErrorCode=ERRORCODE_NULLPARAMETER;
    MESSAGELOG_LogError(ErrorCode);
  }
  else
    ErrorCode=pMoonAnimation->Images.count();

  DEBUGLOG_LogOut();
  return(ErrorCode);
}

ERRORCODE_T MoonAnimation_ReadFile(
    MOONANIMATION_T *pMoonAnimation,char const *pPathname)
{
  ERRORCODE_T ErrorCode;
  QPixmap Image;
  int FrameCount;
  int DX;
  int DY;
  int Size;
  int Frame;


  DEBUGLOG_Printf3(
      "MoonAnimation_ReadFile(%p,%p(%s))",pMoonAnimation,pPathname,pPathname);
  DEBUGLOG_LogIn();

  /* Parameter checking. */
  if ( (pMoonAnimation==NULL) || (pPathname==NULL) )
  {
    ErrorCode=ERRORCODE_NULLPARAMETER;
    MESSAGELOG_LogError(ErrorCode);
  }
  else
  {
    pMoonAnimation->Images.clear();
    ErrorCode=ERRORCODE_INVALIDDATA;
    if (Image.load(pPathname)==false)
      ErrorCode=ERRORCODE_LIBRARYFAILURE;
    else
    {
      FrameCount=0;
      if (Image.isNull()==0)
      {
        /* Just to shut up the MSVC compiler. */
        DX=0;
        DY=0;
        Size=0;

        /* How are the frames arranged? */
        if (Image.width()>=Image.height())
        {
          /* Horizontally. Make sure the width is a multiple of the height. */
          if (Image.width()%Image.height()==0)
          {
            FrameCount=Image.width()/Image.height();
            DX=Image.height();
            DY=0;
            Size=Image.height();
          }
        }
        else
        {
          /* Vertically. Make sure the height is a multiple of the width. */
          if (Image.height()%Image.width()==0)
          {
            FrameCount=Image.height()/Image.width();
            DX=0;
            DY=Image.width();
            Size=Image.width();
          }
        }

        /* Valid image? */
        if (FrameCount>0)
        {
          /* Split the image into frames. */
          for(Frame=0;Frame<FrameCount;Frame++)
            pMoonAnimation->Images.append(
                Image.copy(Frame*DX,Frame*DY,Size,Size));
          ErrorCode=ERRORCODE_SUCCESS;
        }
      }
    }
  }

  DEBUGLOG_LogOut();
  return(ErrorCode);
}


#undef    MOONANIMATION_CPP


/**
*** moonanimation.cpp
**/
